;**** **** **** **** ****
;
;Die Benutzung der Software ist mit folgenden Bedingungen verbunden:
;
;1. Da ich alles kostenlos zur Verfgung stelle, gebe ich keinerlei Garantie
;   und bernehme auch keinerlei Haftung fr die Folgen der Benutzung.
;
;2. Die Software ist ausschlielich zur privaten Nutzung bestimmt. Ich
;   habe nicht geprft, ob bei gewerblicher Nutzung irgendwelche Patentrechte
;   verletzt werden oder sonstige rechtliche Einschrnkungen vorliegen.
;
;3. Jeder darf nderungen vornehmen, z.B. um die Funktion seinen Bedrfnissen
;   anzupassen oder zu erweitern. Ich wrde mich freuen, wenn ich weiterhin als
;   Co-Autor in den Unterlagen erscheine und mir ein Link zur entprechenden Seite
;   (falls vorhanden) mitgeteilt wird.
;
;4. Auch nach den nderungen sollen die Software weiterhin frei sein, d.h. kostenlos bleiben.
;
;!! Wer mit den Nutzungbedingungen nicht einverstanden ist, darf die Software nicht nutzen !!
;
; tp-18a
; October 2004
; autor: Bernhard Konze
; email: bernhard.konze@versanet.de
;--
; Forked from "TGYP2010_416HzRCIntervalRate_PPM-Mod-only_NoCal" which removed
; OSCCAL setting from EEPROM and tweaked some startup parameters, all which
; have since been replaced.
;
; NO WARRANTY EXPRESSED OR IMPLIED. USE AT YOUR OWN RISK. Always test
; without propellers! Please respect Bernhard Konze's license above.
;--
; WARNING: I have blown FETs on Turnigy Plush 18A ESCs in previous versions
; of this code with my modifications. Some bugs have since been fixed, such
; as leaving PWM enabled while busy-looping forever outside of ISR code.
; However, this does run with higher PWM frequency than most original code,
; so higher FET temperatures may occur! USE AT YOUR OWN RISK, and maybe see
; how it compares and let me know!
;
; WARNING: This does not check temperature or voltage ADC inputs.
;
; NOTE: We do 16-bit PWM on timer2 at the full CPU clock rate, using tcnt2h
; to simulate the high byte. An input FULL to STOP range of 800 plus a
; MIN_DUTY of 63 (a POWER_RANGE of 863) gives 800 unique PWM steps at an
; about 18kHz on a 16MHz CPU clock. The output frequency is slightly lower
; than F_CPU / POWER_RANGE due to cycles used in the interrupt before
; reloading TCNT2.
;
; Simon Kirby <sim@simulated.ca>
;
;-- Device ----------------------------------------------------------------
;
.include "m8def.inc"
;
; 8K Bytes of In-System Self-Programmable Flash
; 512 Bytes EEPROM
; 1K Byte Internal SRAM
;
;-- Fuses -----------------------------------------------------------------
;
; Old fuses for internal RC oscillator at 8 MHz were lfuse=0xa4 hfuse=0xdf,
; but since we now set OSCCAL to actually run at about 16 MHz, we'd better
; set brown-out detection to 4.0V. This code should work without changes on
; boards with external 16MHz crystals / resonators; just set lfuse=0x3f.
;
; Suggested fuses with 4.0V brown-out voltage:
; Without external crystal: avrdude -U lfuse:w:0x24:m -U hfuse:w:0xd7:m
;    With external crystal: avrdude -U lfuse:w:0x3f:m -U hfuse:w:0xd7:m
;
; Testing fuses with 2.7V brown-out voltage (unsafe at 16MHz):
; Without external crystal: avrdude -U lfuse:w:0xa4:m -U hfuse:w:0xd7:m
;    With external crystal: avrdude -U lfuse:w:0xbf:m -U hfuse:w:0xd7:m
;
;-- Board -----------------------------------------------------------------
;
; The following only works with avra or avrasm2.
; For avrasm32, just comment out all but the include you need.
.if defined(afro_esc)
.include "afro.inc"		; AfroESC (ICP PPM)
.elif defined(bs_esc)
.include "bs.inc"		; HobbyKing BlueSeries / Mystery (INT0 PPM)
.elif defined(bs_nfet_esc)
.include "bs_nfet.inc"		; HobbyKing BlueSeries / Mystery with all nFETs (INT0 PPM)
.elif defined(rct50a_esc)
.include "rct50a.inc"		; RCTimer 50A with all nFETs (INT0 PPM)
.elif defined(tp_esc)
.include "tp.inc"		; TowerPro 25A/HobbyKing 18A "type 1" (INT0 PPM)
.elif defined(tp_nfet_esc)
.include "tp_nfet.inc"		; TowerPro 25A with all nFETs "type 3" (INT0 PPM)
.elif defined(tgy6a_esc)
.include "tgy6a.inc"		; Turnigy Plush 6A (INT0 PPM)
.else
.include "tgy.inc"		; TowerPro/Turnigy Basic/Plush "type 2" (INT0 PPM)
.endif

.equ	TIME_ACCUMULATE	= 0	; Accumulate 4 commutations timing method
.equ	TIME_HALFADD	= 0	; Update half timing method
.equ	TIME_QUARTERADD	= 1	; Update quarter timing method (original)

.equ	MOTOR_BRAKE	= 0	; Enable brake
.equ	MOTOR_REVERSE	= 0	; Reverse normal commutation direction
.equ	RC_PULS_REVERSE	= 0	; Enable RC-car style forward/reverse throttle
.equ	SLOW_THROTTLE	= 0	; Limit maximum throttle jump to try to prevent overcurrent

.equ	RCP_TOT		= 16	; Number of 65536us periods before considering rc pulse lost
.equ	CPU_MHZ		= F_CPU / 1000000

.if defined(ultrapwm)
.equ	STOP_RC_PULS	= 200	; Support for http://www.xaircraft.com/wiki/UltraPWM/en
.equ	FULL_RC_PULS	= 1200	; which says motors should start at 200us,
.equ	MAX_RC_PULS	= 1400	; but does not define min/max pulse width.
.elif RC_PULS_REVERSE
.equ	STOP_RC_PULS	= 1000	; Stop motor at or below this pulse length
.equ	FULL_RC_PULS	= 1900	; Full speed at or above this pulse length
.equ	MAX_RC_PULS	= 2200	; Throw away any pulses longer than this
.else
.equ	STOP_RC_PULS	= 1060	; Stop motor at or below this pulse length
.equ	FULL_RC_PULS	= 1860	; Full speed at or above this pulse length
.equ	MAX_RC_PULS	= 2200	; Throw away any pulses longer than this
.endif

.if	RC_PULS_REVERSE
.equ	NEUTRAL_RC_PULS	= FULL_RC_PULS + STOP_RC_PULS	; Doubled
.equ	RCP_DEADBAND	= 100	; Do not start until this much above or below neutral
.equ	RCP_DIVISOR	= CPU_MHZ / 2 ; Work with half-microsecond steps
.else
.equ	NEUTRAL_RC_PULS	= STOP_RC_PULS
.equ	RCP_DEADBAND	= 0
.equ	RCP_DIVISOR	= CPU_MHZ ; Work with microsecond steps
.endif

.equ	MIN_DUTY	= 63	; Minimum PWM on-time (too low and FETs won't turn on, hard starting)
.equ	POWER_RANGE	= FULL_RC_PULS - STOP_RC_PULS - RCP_DEADBAND + MIN_DUTY

.equ	MAX_POWER	= (POWER_RANGE-1)
.equ	PWR_MIN_START	= (POWER_RANGE/6) ; Power limit until running mode
.equ	PWR_MAX_START	= (POWER_RANGE/4) ; Power limit until running mode
.equ	PWR_MAX_RPM1	= (POWER_RANGE/4) ; Power limit when running slower than TIMING_RANGE1
.equ	PWR_MAX_RPM2	= (POWER_RANGE/2) ; Power limit when running slower than TIMING_RANGE2

.equ	TIMING_MIN	= 0x8000 ; 8192us per commutation
.equ	TIMING_RUN	= 0x1000 ; 1024us per commutation
.equ	TIMING_RANGE1	= 0x4000 ; 4096us per commutation
.equ	TIMING_RANGE2	= 0x2000 ; 2048us per commutation
.equ	TIMING_MAX	= 0x0050 ; 20us per commutation

.equ	timeoutSTART	= 48000 ; 48ms per commutation
.equ	timeoutMIN	= 36000	; 36ms per commutation

.equ	ENOUGH_GOODIES	= 12	; This many start cycles without timeout will transition to running mode

.equ	T0CLK		= (1<<CS01)	; clk/8 == 2Mhz
.equ	T1CLK		= (1<<CS10)	; clk/1 == 16MHz
.equ	T2CLK		= (1<<CS20)	; clk/1 == 16MHz

;**** **** **** **** ****
; Register Definitions
.def	zero		= r0		; stays at 0
.def	i_sreg		= r1		; status register save in interrupts
.def	duty_l		= r2		; on duty cycle low, one's complement
.def	duty_h		= r3		; on duty cycle high
.def	off_duty_l	= r4		; off duty cycle low, one's complement
.def	off_duty_h	= r5		; off duty cycle high
.def	rcpuls_l	= r6
.def	rcpuls_h	= r7
.def	tcnt2h		= r8
.def	temp5		= r9		; aux temporary (limited operations)
.def	uart_cnt	= r10
.def	rc_timeout	= r11
.def	sys_control_l	= r12		; duty limit low (word register aligned)
.def	sys_control_h	= r13		; duty limit high
;.def	acsr_save	= r14		; saved ACSR register value

.def	temp1		= r16		; main temporary (L)
.def	temp2		= r17		; main temporary (H)
.def	temp3		= r18		; main temporary (L)
.def	temp4		= r19		; main temporary (H)

.def	i_temp1		= r20		; interrupt temporary
.def	i_temp2		= r21		; interrupt temporary

.def	nfet_on		= r22
.def	nfet_off	= r25

.def	flags0	= r23	; state flags
	.equ	OCT1_PENDING	= 0	; if set, output compare interrupt is pending
	.equ	OCT1B_PENDING	= 1	; if set, output compare interrupt B is pending
;	.equ	I_pFET_HIGH	= 2	; set if over-current detect
;	.equ	GET_STATE	= 3	; set if state is to be send
;	.equ	C_FET		= 4	; if set, C-FET state is to be changed
;	.equ	A_FET		= 5	; if set, A-FET state is to be changed
;	.equ	I_FET_ON	= 6	; if set, fets off
;	.equ	I_ON_CYCLE	= 7	; if set, current on cycle is active (optimized as MSB)

.def	flags1	= r24	; state flags
	.equ	POWER_OFF	= 0	; switch fets on disabled
	.equ	FULL_POWER	= 1	; 100% on - don't switch off, but do OFF_CYCLE working
	.equ	I2C_MODE	= 2	; if receiving updates via I2C
;	.equ	RC_PULS_UPDATED	= 3	; rcpuls value has changed
	.equ	EVAL_RC		= 4	; if set, evaluate rc command while waiting for OCT1
;	.equ    EVAL_SYS_STATE	= 5	; if set, overcurrent and undervoltage are checked
	.equ    STARTUP		= 6	; if set, startup-phase is active
	.equ	REVERSE		= 7	; if set, do reverse commutation

;.def	flags2	= r25
;	.equ	RPM_RANGE1	= 0	; if set RPM is lower than 1831 RPM
;	.equ	RPM_RANGE2	= 1	; if set RPM is lower than 3662 RPM
;	.equ	RC_INTERVAL_OK	= 2
;	.equ	POFF_CYCLE	= 3	; if set one commutation cycle is performed without power
;	.equ	COMP_SAVE	= 4	; if set ACO was high
;	.equ	COMP_SAVE_READY	= 5	; if acsr_save was set by PWM interrupt
;	.equ	STARTUP		= 6	; if set startup-phase is active
;	.equ	SCAN_TIMEOUT	= 7	; if set a startup timeout occurred

; here the XYZ registers are placed ( r26-r31)

; XL: I/O address of PWM nFET port
; XH: I/O address of PWM nFET port
; YL: general temporary
; YH: general temporary
; ZL: Next PWM interrupt vector (low)
; ZH: Next PWM interrupt vector (high, stays at zero)

;**** **** **** **** ****
; RAM Definitions
.dseg				; DATA segment
.org SRAM_START

last_tcnt1_l:	.byte	1	; last timer1 value
last_tcnt1_h:	.byte	1
last_tcnt1_x:	.byte	1
timing_l:	.byte	1	; holds time of 4 commutations
timing_h:	.byte	1
timing_x:	.byte	1
timing_count:	.byte	1

wt_comp_scan_l:	.byte	1	; time from switch to comparator scan (blanking time)
wt_comp_scan_h:	.byte	1
wt_comp_scan_x:	.byte	1
com_timing_l:	.byte	1	; time from zero-crossing to switch of the appropriate FET
com_timing_h:	.byte	1
com_timing_x:	.byte	1
wt_OCT1_tot_l:	.byte	1	; time for each startup commutation
wt_OCT1_tot_h:	.byte	1
wt_OCT1_tot_x:	.byte	1
zero_wt_l:	.byte	1	; time to wait for zero-crossing while running
zero_wt_h:	.byte	1
zero_wt_x:	.byte	1
zc_filter_time:	.byte	1	; number of times to check zero-crossing

ocr1ax:		.byte	1	; 3rd byte of OCR1A
ocr1bx:		.byte	1	; 3rd byte of OCR1B
tcnt1x:		.byte	1	; 3rd byte of TCNT1

start_rcpuls_l:	.byte	1
start_rcpuls_h:	.byte	1
start_rcpuls_x:	.byte	1
rc_duty_l:	.byte	1	; desired duty cycle
rc_duty_h:	.byte	1
timing_duty_l:	.byte	1	; duty cycle limit based on timing
timing_duty_h:	.byte	1
max_power_l:	.byte	1
max_power_h:	.byte	1
neutral_l:	.byte	1
neutral_h:	.byte	1

goodies:	.byte	1

;**** **** **** **** ****
; ATmega8 interrupts

;.equ	INT0addr=$001	; External Interrupt0 Vector Address
;.equ	INT1addr=$002	; External Interrupt1 Vector Address
;.equ	OC2addr =$003	; Output Compare2 Interrupt Vector Address
;.equ	OVF2addr=$004	; Overflow2 Interrupt Vector Address
;.equ	ICP1addr=$005	; Input Capture1 Interrupt Vector Address
;.equ	OC1Aaddr=$006	; Output Compare1A Interrupt Vector Address
;.equ	OC1Baddr=$007	; Output Compare1B Interrupt Vector Address
;.equ	OVF1addr=$008	; Overflow1 Interrupt Vector Address
;.equ	OVF0addr=$009	; Overflow0 Interrupt Vector Address
;.equ	SPIaddr =$00a	; SPI Interrupt Vector Address
;.equ	URXCaddr=$00b	; USART Receive Complete Interrupt Vector Address
;.equ	UDREaddr=$00c	; USART Data Register Empty Interrupt Vector Address
;.equ	UTXCaddr=$00d	; USART Transmit Complete Interrupt Vector Address
;.equ	ADCCaddr=$00e	; ADC Interrupt Vector Address
;.equ	ERDYaddr=$00f	; EEPROM Interrupt Vector Address
;.equ	ACIaddr =$010	; Analog Comparator Interrupt Vector Address
;.equ	TWIaddr =$011	; Irq. vector address for Two-Wire Interface
;.equ	SPMaddr =$012	; SPM complete Interrupt Vector Address
;.equ	SPMRaddr =$012	; SPM complete Interrupt Vector Address
;-----bko-----------------------------------------------------------------

;**** **** **** **** ****
.cseg
.org 0
;**** **** **** **** ****

;-----bko-----------------------------------------------------------------
; Reset and interrupt jump table
; When multiple interrupts are pending, the vectors are executed from top
; (ext_int0) to bottom.
		rjmp reset
		rjmp_ext_int0	; ext_int0
		nop		; ext_int1
		nop		; t2oc_int
		ijmp		; t2ovfl_int
		rjmp_icp1_int	; icp1_int
		rjmp t1oca_int
		rjmp t1ocb_int	; t1ocb_int
		rjmp t1ovfl_int
		nop		; t0ovfl_int
		nop		; spi_int
		nop		; urxc
		nop		; udre
		nop		; utxc

; not used	nop		; adc_int
; not used	nop		; eep_int
; not used	nop		; aci_int
; not used	nop		; wire2_int
; not used	nop		; spmc_int

;-----bko-----------------------------------------------------------------
; init after reset

reset:		clr	r0
		out	SREG, r0		; Clear interrupts and flags

	; Set up stack
		ldi	ZH, high(RAMEND)
		ldi	ZL, low (RAMEND)
		out	SPH, ZH
		out	SPL, ZL
	; clear RAM and all registers
clear_loop:	st	-Z, r0
		cpi	ZL, SRAM_START
		cpc	ZH, r0
		brne	clear_loop1
		ldi	ZL, 30			; Start clearing registers
clear_loop1:	cp	ZL, r0
		cpc	ZH, r0
		brne	clear_loop

	; portB - all FETs off
		ldi	temp1, INIT_PB		; PORTB initially holds 0x00
		out	PORTB, temp1
		ldi	temp1, DIR_PB
		out	DDRB, temp1

	; portC reads comparator inputs
		ldi	temp1, INIT_PC
		out	PORTC, temp1
		ldi	temp1, DIR_PC
		out	DDRC, temp1

	; portD reads rc-puls + AIN0 ( + RxD, TxD for debug )
		ldi	temp1, INIT_PD
		out	PORTD, temp1
		ldi	temp1, DIR_PD
		out	DDRD, temp1

	; Start timers except output PWM
		ldi	temp1, T0CLK		; timer0: beep control, delays
		out	TCCR0, temp1
		ldi	temp1, T1CLK+T1ICP	; timer1: commutation timing,
		out	TCCR1B, temp1		; RC pulse measurement
		out	TCCR2, zero		; timer2: PWM, stopped

	; Set PWM interrupt vector
		ldi	ZL, low(pwm_off)	; Set PWM interrupt vector

	; Set default max_power and neutral levels
	; (eventually, these should be loaded from EEPROM)
		ldi	temp1, byte1(MAX_POWER)
		sts	max_power_l, temp1
		ldi	temp1, byte2(MAX_POWER)
		sts	max_power_h, temp1
		ldi	temp1, byte1(NEUTRAL_RC_PULS)
		sts	neutral_l, temp1
		ldi	temp1, byte2(NEUTRAL_RC_PULS)
		sts	neutral_h, temp1

	; Set clock to almost 16MHz -- this should be stable provided we do
	; not write to the EEPROM or flash unless we revert OSCCAL to 0x9f
		rcall	wait260ms	; wait a while
.if CPU_MHZ == 16
		ldi	temp1, 0xff
		out	OSCCAL, temp1
.elif CPU_MHZ == 8
		ldi	temp1, 0x9f
		out	OSCCAL, temp1
.endif
		rcall	wait30ms

	; Check reset cause
		in	i_sreg, MCUCSR
		out	MCUCSR, zero

		sbrs	i_sreg, PORF		; Power-on reset
		rjmp	init_no_porf
		rcall	beep_f1			; Usual startup beeps
		rcall	wait30ms
		rcall	beep_f2
		rcall	wait30ms
		rcall	beep_f3
		rjmp	control_start

init_no_porf:	sbrs	i_sreg, BORF		; Brown-out reset
		rjmp	init_no_borf
		rcall	beep_f3			; "dead cellphone"
		rcall	beep_f1

init_no_borf:	sbrs	i_sreg, EXTRF	; External reset
		rjmp	control_start
		rcall	beep_f4			; Single beep

control_start:
		rcall	wait260ms

		; status led on
		GRN_on

	; init registers and interrupts
		ldi	temp1, (1<<TOIE1)+(1<<OCIE1A)+(1<<OCIE1B)+(1<<OCIE2)
		out	TIFR, temp1		; clear TOIE1,OCIE1A & OCIE2
		out	TIMSK, temp1		; enable TOIE1,OCIE1A & OCIE2 interrupts

		sei				; enable all interrupts

	; init rc-puls
		rcp_int_rising_edge temp1
		rcp_int_enable temp1
i_rc_puls1:	clr	rc_timeout
i_rc_puls2:	sbrs	flags1, EVAL_RC
		rjmp	i_rc_puls2
		rcall	evaluate_rc
		lds	YL, rc_duty_l
		lds	YH, rc_duty_h
		adiw	YL, 0			; Test for zero
		brne	i_rc_puls1
		ldi	temp1, 10		; wait for this count of receiving power off
		cp	rc_timeout, temp1
		brlo	i_rc_puls2
		cli				; disable all interrupts
		rcall	beep_f4			; signal: rcpuls ready
		rcall	beep_f4
		rcall	beep_f4
		rjmp	init_startup

;-----bko-----------------------------------------------------------------
; NOTE: This interrupt uses the 16-bit atomic timer read/write register
; by reading TCNT1L and TCNT1H, so this interrupt must be disabled before
; any other 16-bit timer options happen that might use the same register
; (see "Accessing 16-bit registers" in the Atmel documentation)
.if USE_ICP
; icp1 = rc pulse input, if enabled
icp1_int:	in	i_temp1, ICR1L		; get captured timer values
		in	i_temp2, ICR1H
		in	i_sreg, TCCR1B		; abuse i_sreg to hold value
		sbrs	i_sreg, ICES1		; evaluate edge of this interrupt
.else
;-----bko-----------------------------------------------------------------
ext_int0:	in	i_temp1, TCNT1L		; get timer1 values
		in	i_temp2, TCNT1H
		sbis	PIND, rcp_in		; evaluate edge of this interrupt
.endif
		rjmp	falling_edge		; bit is clear = falling edge
rising_edge:					; Flags not saved here!
		sts	start_rcpuls_l, i_temp1	; Save pulse start time
		rcp_int_falling_edge i_temp1	; Set next int to falling edge
		sts	start_rcpuls_h, i_temp2
		lds	i_temp1, tcnt1x
		sts	start_rcpuls_x, i_temp1
		sbrc	i_temp2, 7
		reti
		in	i_temp2, TIFR
		sbrs	i_temp2, TOV1
		reti
		in	i_sreg, SREG
		inc	i_temp1			; Compensate for postponed tcnt1x update
		sts	start_rcpuls_x, i_temp1
		out	SREG, i_sreg
		reti

rcpint_fail:	cpse	rc_timeout, zero
		dec	rc_timeout
		rjmp	rcpint_exit

falling_edge:
		in	i_sreg, SREG
		rcp_int_rising_edge XH		; Set next int to rising edge
		lds	XH, tcnt1x		; We borrow XH and ZH (normally 0)
		sbrc	i_temp2, 7		; as additional registers and
		rjmp	falling_edge1		; clear them before returning.
		in	ZH, TIFR
		sbrc	ZH, TOV1
		inc	XH			; Compensate for postponed tcnt1x update
falling_edge1:	lds	ZH, start_rcpuls_l
		sub	i_temp1, ZH
		lds	ZH, start_rcpuls_h
		sbc	i_temp2, ZH
		lds	ZH, start_rcpuls_x
		sbc	XH, ZH

.if byte3(MAX_RC_PULS*CPU_MHZ)
.error "MAX_RC_PULS too high: adjust it or the pulse length checking code"
.endif
		cpi	i_temp1, byte1(MAX_RC_PULS*CPU_MHZ)
		ldi	ZH, byte2(MAX_RC_PULS*CPU_MHZ)
		cpc	i_temp2, ZH
		ldi	ZH, 0			; Return ZH to 0; clr clobbers flags
		cpc	XH, ZH
		ldi	XH, 0			; Return XH to 0
		brsh	rcpint_fail		; throw away (too long pulse)

		movw	rcpuls_l, i_temp1
		sbr	flags1, (1<<EVAL_RC)

rcpint_exit:	out	SREG, i_sreg
		reti
;-----bko-----------------------------------------------------------------
; timer output compare interrupt
t1oca_int:	in	i_sreg, SREG
		lds	i_temp1, ocr1ax
		subi	i_temp1, 1
		brcc	t1oca_int1
		cbr	flags0, (1<<OCT1_PENDING)	; signal OCT1A passed
t1oca_int1:	sts	ocr1ax, i_temp1
		out	SREG, i_sreg
		reti
;-----bko-----------------------------------------------------------------
; timer output compare B interrupt
t1ocb_int:	in	i_sreg, SREG
		lds	i_temp1, ocr1bx
		subi	i_temp1, 1
		brcc	t1ocb_int1
		cbr	flags0, (1<<OCT1B_PENDING)	; signal OCT1B passed
t1ocb_int1:	sts	ocr1bx, i_temp1
		out	SREG, i_sreg
		reti
;-----bko-----------------------------------------------------------------
; timer1 overflow interrupt (happens every 4096s)
t1ovfl_int:	in	i_sreg, SREG
		lds	i_temp1, tcnt1x
		inc	i_temp1
		sts	tcnt1x, i_temp1
		andi	i_temp1, 15			; Every 16 overflows
		brne	t1ovfl_int1
		cpse	rc_timeout, zero
		dec	rc_timeout
t1ovfl_int1:	out	SREG, i_sreg
		reti
;-----bko-----------------------------------------------------------------
; timer2 overflow compare interrupt (output PWM) -- the interrupt vector
; actually "ijmp"s to Z which should point to one of these entry points.
;
; We try to avoid clobbering (and thus needing to save/restore) flags;
; in, out, mov, ldi, etc. do not modify any flags, while dec does.
;
; The comparator (ACSR) is saved at the very end of the ON cycle, but
; since the nFET takes at least half a microsecond to turn off and the
; AVR buffers ACO for a few cycles, we do it after turning off the drive
; pin. For low duty cycles (with a longer off period), testing shows that
; waiting an extra 0.5us - 0.75us (8-12 cycles at 16MHz) actually helps
; to improve zero-crossing detection accuracy significantly, perhaps
; because the driven-low phase has had a chance to finish swinging down.
; However, some tiny boards such as 10A or less may have very low gate
; charge/capacitance, and so can turn off faster. We used to wait 8/9
; cycles, but now we wait 5 cycles (5/16ths of a microsecond), which
; still helps on ~30A boards without breaking 10A boards.
;
; We reload TCNT2 as the very last step so as to reduce PWM dead areas
; between the reti and the next interrupt vector execution, which still
; takes a good 4 (reti) + 4 (interrupt call) + 2 (ijmp) cycles. We also
; try to keep the fet switch off as close to this as possible to avoid a
; significant bump at FULL_POWER.
;
; The pwm_*_high entry points are only called when the particular on/off
; cycle is longer than 8 bits. This is tracked in tcnt2h.

pwm_on_high:
		in	i_sreg, SREG
		dec	tcnt2h
		brne	pwm_on_again
		ldi	ZL, pwm_on
pwm_on_again:	out	SREG, i_sreg
		reti
pwm_off_high:
		in	i_sreg, SREG
		dec	tcnt2h
		brne	pwm_off_again
		ldi	ZL, pwm_off
pwm_off_again:	out	SREG, i_sreg
		reti

pwm_on:
		st	X, nfet_on
		ldi	ZL, pwm_off
		cpse	duty_h, zero
		ldi	ZL, pwm_off_high
		mov	tcnt2h, duty_h
		out	TCNT2, duty_l
		reti

pwm_off:
		ldi	ZL, pwm_on		; 1 cycle
		cpse	off_duty_h, zero	; 1 cycle if not zero, 2 if zero
		ldi	ZL, pwm_on_high		; 1 cycle
		mov	tcnt2h, off_duty_h	; 1 cycle
		st	X, nfet_off		; 2 cycles (off at 6 cycles from entry)
		out	TCNT2, off_duty_l	; 1 cycle
		reti				; 4 cycles

.if high(pwm_off)
.error "high(pwm_off) is non-zero; please move code closer to start or use 16-bit (ZH) jump registers"
.endif
;-----bko-----------------------------------------------------------------
; beeper: timer0 is set to 1s/count
beep_f1:	ldi	temp4, 200
		ldi	temp2, 80
		BpFET_on
		AnFET_on
		rjmp	beep

beep_f2:	ldi	temp4, 180
		ldi	temp2, 100
		CpFET_on
		BnFET_on
		rjmp	beep

beep_f3:	ldi	temp4, 160
		ldi	temp2, 120
		ApFET_on
		CnFET_on
		rjmp	beep

beep_f4:	ldi	temp4, 140
		ldi	temp2, 140
		CpFET_on
		AnFET_on
		rjmp	beep

; Interrupts must be disabled before entry
beep:		in	i_temp1, PORTB		; Save ON state
		in	i_temp2, PORTC
		in	YH, PORTD
beep_on:	out	PORTB, i_temp1		; Restore ON state
		out	PORTC, i_temp2
		out	PORTD, YH
		out	TCNT0, zero
beep_BpCn10:	in	temp1, TCNT0
		cpi	temp1, 2*CPU_MHZ	; 32s on
		brlo	beep_BpCn10
		all_nFETs_off YL
		all_pFETs_off YL
		ldi	temp3, CPU_MHZ		; 2040s off
beep_BpCn12:	out	TCNT0, zero
beep_BpCn13:	in	temp1, TCNT0
		cp	temp1, temp4
		brlo	beep_BpCn13
		dec	temp3
		brne	beep_BpCn12
		dec	temp2
		brne	beep_on
		ret

wait30ms:	ldi	temp2, 15
beep_BpCn20:	ldi	temp3, CPU_MHZ
beep_BpCn21:	out	TCNT0, zero
		out	TIFR, zero
beep_BpCn22:	in	temp1, TIFR
		sbrs	temp1, TOV0
		rjmp	beep_BpCn22
		dec	temp3
		brne	beep_BpCn21
		dec	temp2
		brne	beep_BpCn20
		ret

	; 128 periods = 261ms silence
wait260ms:	ldi	temp2, 128
beep2_BpCn20:	ldi	temp3, CPU_MHZ
beep2_BpCn21:	out	TCNT0, zero
beep2_BpCn22:	in	temp1, TCNT0
		cpi	temp1, 200
		brlo	beep2_BpCn22
		dec	temp3
		brne	beep2_BpCn21
		dec	temp2
		brne	beep2_BpCn20
		ret
;-----bko-----------------------------------------------------------------
evaluate_rc:	cbr	flags1, (1<<EVAL_RC)
		sbrc	flags1, I2C_MODE
		rjmp	evaluate_rc_i2c
;-----bko-----------------------------------------------------------------
evaluate_rc_puls:				; No clobbering temp4 here, please
		ldi	temp1, RCP_TOT
		cp	rc_timeout, temp1
		adc	rc_timeout, zero	; Increment if not at RCP_TOT
		movw	YL, rcpuls_l		; Atomic copy of rc pulse length
.if RCP_DIVISOR == 16
		swap	YL			; Divide by 16 (MHz -> us)
		swap	YH
		andi	YL, 0x0f
		eor	YL, YH
		andi	YH, 0x0f
		eor	YL, YH
.elif RCP_DIVISOR == 8
		lsr	YH			; Divide by 8 (us / Mhz / 2 for reverse)
		ror	YL
		lsr	YH
		ror	YL
		lsr	YH
		ror	YL
.elif RCP_DIVISOR == 4
		lsr	YH			; Divide by 8 (us / Mhz / 2 for reverse)
		ror	YL
		lsr	YH
		ror	YL
.else
.error Teach me how to divide by RCP_DIVISOR
.endif
.if MOTOR_REVERSE
		sbr	flags1, (1<<REVERSE)
.else
		cbr	flags1, (1<<REVERSE)
.endif
		lds	temp1, neutral_l
		lds	temp2, neutral_h
		sub	YL, temp1
		sbc	YH, temp2
		brcc	puls_plus
.if RC_PULS_REVERSE
.if MOTOR_REVERSE
		cbr	flags1, (1<<REVERSE)
.else
		sbr	flags1, (1<<REVERSE)
.endif
		subi	YL, -low(RCP_DEADBAND)
		sbci	YH, -1 - high(RCP_DEADBAND)
		brpl	puls_zero
		com	YH
		neg	YL
		sbci	YH, -1
		rjmp	puls_not_zero
.endif
puls_zero:	clr	YL
		clr	YH
		rjmp	puls_not_full
puls_plus:
.if RCP_DEADBAND
		subi	YL, low(RCP_DEADBAND)
		sbci	YH, high(RCP_DEADBAND)
		brmi	puls_zero
.endif
puls_not_zero:	adiw	YL, MIN_DUTY
		lds	temp1, max_power_l
		lds	temp2, max_power_h
		cp	YL, temp1
		cpc	YH, temp2
		brlo	puls_not_full
		movw	YL, temp1
puls_not_full:	sts	rc_duty_l, YL
		sts	rc_duty_h, YH
		rjmp	set_new_duty_l		; Skip reload into YL:YH
;-----bko-----------------------------------------------------------------
evaluate_rc_i2c:
	; Stub for now
		ret
;-----bko-----------------------------------------------------------------
update_timing:
		lds	temp3, tcnt1x
		rcall	set_ocr1a		; Returns TCNT1L/H in temp1, temp2
		sbrs	temp2, 7		; Unless highest bit of TCNT1H is set,
		lds	temp3, tcnt1x		; load possibly-updated extended byte.

.if TIME_ACCUMULATE
		lds	temp4, timing_count
		cpi	temp4, 4
		brsh	update_timing2
		inc	temp4
		sts	timing_count, temp4
		ret
update_timing2:
.endif
	; calculate this commutation time
		lds	YL, last_tcnt1_l
		lds	YH, last_tcnt1_h
		lds	temp5, last_tcnt1_x
		sts	last_tcnt1_l, temp1
		sts	last_tcnt1_h, temp2
		sts	last_tcnt1_x, temp3
		sub	temp1, YL
		sbc	temp2, YH
		sbc	temp3, temp5

	; calculate next waiting times - timing(-l-h-x) holds the time of 4 commutations

.if TIME_ACCUMULATE
	; Accumulation method: Wait for 4 commutations, and that's all.
		movw	YL, temp1		; Copy new timing to YL:YH:temp5
		mov	temp5, temp3
.else
		lds	YL, timing_l		; Load old timing into YL:YH:temp5
		lds	YH, timing_h
		lds	temp5, timing_x

.if TIME_HALFADD
	; Half method: New timing is sum of half of old and twice of last commutation time
		lsr	temp5
		ror	YH
		ror	YL
		lsl	temp1
		rol	temp2
		rol	temp3
		add	YL, temp1
		adc	YH, temp2
		adc	temp5, temp3
.elif TIME_QUARTERADD
	; Quarter method: Subtract quart from old timing, add new commutation time
		lsr	temp5			; build a quater
		ror	YH
		ror	YL
		lsr	temp5
		ror	YH
		ror	YL
		sub	temp1, YL		; subtract old quarter from new timing
		sbc	temp2, YH
		sbc	temp3, temp5
		lds	YL, timing_l
		lds	YH, timing_h
		lds	temp5, timing_x
		add	YL, temp1		; add the new timing difference
		adc	YH, temp2
		adc	temp5, temp3
.endif
.endif
	; Limit maximum RPM (fastest timing)
		cpi	YL, byte1(TIMING_MAX*CPU_MHZ)
		ldi	temp4, byte2(TIMING_MAX*CPU_MHZ)
		cpc	YH, temp4
		ldi	temp4, byte3(TIMING_MAX*CPU_MHZ)
		cpc	temp5, temp4
		brcc	update_timing3
		lsr	sys_control_h		; limit by reducing power
		ror	sys_control_l
		rjmp	update_timing4
update_timing3:
	; With higher KV motors or high voltage, we need to make
	; sure we don't call sync_with_poweron with fast timing,
	; or we might miss.
		cpi	YH, byte2(TIMING_RUN*CPU_MHZ)
		ldi	temp4, byte3(TIMING_RUN*CPU_MHZ)
		cpc	temp5, temp4
		brcc	update_timing4
		cbr	flags1, (1<<STARTUP)
update_timing4:
	; Limit minimum RPM (slowest timing)
		ldi	temp4, byte3(TIMING_MIN*CPU_MHZ)
		cp	temp5, temp4
		brcs	update_timing5
		mov	temp5, temp4
		ldi	YH, byte2(TIMING_MIN*CPU_MHZ)
		ldi	YL, byte1(TIMING_MIN*CPU_MHZ)
update_timing5:
.if TIME_ACCUMULATE
		sts	timing_count, zero
.endif
		sts	timing_l, YL		; save new timing
		sts	timing_h, YH
		sts	timing_x, temp5

		sts	zero_wt_l, YL		; save zero crossing timeout
		sts	zero_wt_h, YH
		sts	zero_wt_x, temp5

	; Calculate a hopefully sane duty cycle limit from this timing,
	; to prevent excessive current if high duty is requested when the
	; current duty is low. This is the best we can do without a current
	; sensor. The actual current will depend on motor KV and voltage,
	; so this is just an approximation. It would be nice if we could
	; do this with math instead of two constants, but we need a divide.

		ldi	temp1, low(MAX_POWER)
		ldi	temp2, high(MAX_POWER)
		cpi	YH, byte2(TIMING_RANGE2*CPU_MHZ)
		ldi	temp4, byte3(TIMING_RANGE2*CPU_MHZ)
		cpc	temp5, temp4
		brcs	update_timing6
		ldi	temp1, low(PWR_MAX_RPM2)
		ldi	temp2, high(PWR_MAX_RPM2)
		cpi	YH, byte2(TIMING_RANGE1*CPU_MHZ)
		ldi	temp4, byte3(TIMING_RANGE1*CPU_MHZ)
		cpc	temp5, temp4
		brcs	update_timing6
		ldi	temp1, low(PWR_MAX_RPM1)
		ldi	temp2, high(PWR_MAX_RPM1)
update_timing6:	sts	timing_duty_l, temp1	; Save new duty limit by timing
		sts	timing_duty_h, temp2

		mov	temp1, YH		; Copy high and check extended byte
		tst	temp5			; We work with 1/256th of timing
		breq	update_timing7
		ldi	temp1, 0xff
.if TIMING_MAX*CPU_MHZ / 0xff < 3
.error "TIMING_MAX is too fast for at least 3 zero-cross checks -- increase it or adjust this"
.endif
update_timing7:	sts	zc_filter_time, temp1	; Save zero cross filter time

		lsr	temp5			; shift back to timing for one commutation
		ror	YH
		ror	YL
		lsr	temp5
		ror	YH
		ror	YL

		lsr	temp5			; a quarter is the next wait before scan
		ror	YH
		ror	YL
		lsr	temp5
		ror	YH
		ror	YL

		sts	wt_comp_scan_l, YL	; save zero-cross blanking wait time (15)
		sts	wt_comp_scan_h, YH
		sts	wt_comp_scan_x, temp5

		sts	com_timing_l, YL	; use the same value for commutation timing (15)
		sts	com_timing_h, YH
		sts	com_timing_x, temp5

		sbrc	flags1, EVAL_RC
		rjmp	evaluate_rc		; Set new duty either way
		rjmp	set_new_duty
;-----bko-----------------------------------------------------------------
calc_next_timing_and_wait:
		sbrc	flags1, STARTUP
		rjmp	start_timeout

		lds	YL, wt_comp_scan_l	; holds wait-before-scan value
		lds	YH, wt_comp_scan_h
		lds	temp5, wt_comp_scan_x
		rcall	update_timing

wait_OCT1_tot:	sbrc	flags1, EVAL_RC
		rcall	evaluate_rc
		sbrc	flags0, OCT1_PENDING
		rjmp	wait_OCT1_tot

		lds	YL, zero_wt_l
		lds	YH, zero_wt_h
		lds	temp5, zero_wt_x
set_ocr1a:	adiw	YL, 7			; Compensate for timer increment during in-add-out
		ldi	temp4, (1<<OCF1A)
		cli
		in	temp1, TCNT1L
		in	temp2, TCNT1H
		add	YL, temp1
		adc	YH, temp2
		out	OCR1AH, YH
		out	OCR1AL, YL
		out	TIFR, temp4		; Clear any pending OCF1A interrupt (7 cycles from TCNT1 read)
		sts	ocr1ax, temp5
		sbr	flags0, (1<<OCT1_PENDING)
		sei				; We could use reti here, but that's
		ret				; 4 more cycles of interrupt latency
;-----bko-----------------------------------------------------------------
set_com_timing:					; Does not clobber temp4
		lds	YL, com_timing_l
		lds	YH, com_timing_h
		lds	temp5, com_timing_x
set_ocr1b:	adiw	YL, 7			; Compensate for cycles during in-add-out before OCF1B clear
		ldi	temp3, (1<<OCF1B)
		cli
		in	temp1, TCNT1L
		in	temp2, TCNT1H
		add	YL, temp1
		adc	YH, temp2
		out	OCR1BH, YH
		out	OCR1BL, YL
		out	TIFR, temp3		; Clear any pending OCF1B interrupt (7 cycles from TCNT1 read)
		sts	ocr1bx, temp5
		sbr	flags0, (1<<OCT1B_PENDING)
		sei
		ret
;-----bko-----------------------------------------------------------------
start_timeout_start:
		ldi	YL, byte1(timeoutSTART*CPU_MHZ)
		ldi	YH, byte2(timeoutSTART*CPU_MHZ)
		ldi	temp1, byte3(timeoutSTART*CPU_MHZ)
		mov	temp5, temp1
		ret
;-----bko-----------------------------------------------------------------
start_timeout:
		lds	YL, wt_OCT1_tot_l	; Load the start commutation
		lds	YH, wt_OCT1_tot_h	; timeout into YL:YH:temp5 and
		lds	temp5, wt_OCT1_tot_x	; subtract a "random" amount
		in	temp1, TCNT0
		andi	temp1, 0x1f
		sub	YH, temp1
		sbc	temp5, zero
		brcs	start_timeout1
		cpi	YL, byte1(timeoutMIN*CPU_MHZ)
		ldi	temp1, byte2(timeoutMIN*CPU_MHZ)
		cpc	YH, temp1
		ldi	temp1, byte3(timeoutMIN*CPU_MHZ)
		cpc	temp5, temp1
		brcc	start_timeout2
start_timeout1:	rcall	start_timeout_start
start_timeout2:	sts	wt_OCT1_tot_l, YL	; Return with YL:YH:temp5 set
		sts	wt_OCT1_tot_h, YH	; to new wt_OCT1_tot value
		sts	wt_OCT1_tot_x, temp5
		rjmp	update_timing
;-----bko-----------------------------------------------------------------
set_new_duty:	lds	YL, rc_duty_l
		lds	YH, rc_duty_h
set_new_duty_l:	lds	temp1, timing_duty_l
		lds	temp2, timing_duty_h
		cp	YL, temp1
		cpc	YH, temp2
		brcs	set_new_duty10
		movw	YL, temp1		; Limit duty to timing_duty
set_new_duty10:	cp	YL, sys_control_l
		cpc	YH, sys_control_h
		brcs	set_new_duty11
		movw	YL, sys_control_l	; Limit duty to sys_control
set_new_duty11:
.if SLOW_THROTTLE
		; If sys_control is higher than twice the current duty,
		; limit it to that. This means that a steady-state duty
		; cycle can double at any time, but any larger change will
		; be rate-limited.
		ldi	temp1, low(PWR_MIN_START)
		cp	YL, temp1
		ldi	temp2, high(PWR_MIN_START)
		cpc	YH, temp2
		brcs	set_new_duty12
		movw	temp1, YL		; temp1:temp2 >= PWR_MIN_START
set_new_duty12:	lsl	temp1
		rol	temp2
		cp	sys_control_l, temp1
		cpc	sys_control_h, temp2
		brcs	set_new_duty13
		movw	sys_control_l, temp1
set_new_duty13:
.endif
		ldi	temp1, low(MAX_POWER)
		ldi	temp2, high(MAX_POWER)
		sub	temp1, YL		; Calculate OFF duty
		sbc	temp2, YH
		breq	set_new_duty_full
		cbr	flags1, (1<<FULL_POWER)
		cp	YL, zero
		cpc	YH, zero
		breq	set_new_duty_zero
		; Not off and not full power
		; Halve PWM frequency when starting (helps hard drive startup)
		lds	temp3, goodies
		cpi	temp3, ENOUGH_GOODIES
		brcc	set_new_duty_set
		lsl	temp1
		rol	temp2
		lsl	YL
		rol	YH
set_new_duty_set:
		cbr	flags1, (1<<POWER_OFF)
set_new_duty_set_off:
		com	YL			; Save one's complement of both
		com	temp1			; low bytes for up-counting TCNT2
		movw	duty_l, YL		; Atomic set of new ON duty for PWM interrupt
		movw	off_duty_l, temp1	; Atomic set of new OFF duty for PWM interrupt
		ret
set_new_duty_full:
		; Full power
		sbr	flags1, (1<<FULL_POWER)
		rjmp	set_new_duty_set
set_new_duty_zero:
		; Power off
		sbr	flags1, (1<<POWER_OFF)
		rjmp	set_new_duty_set_off
;-----bko-----------------------------------------------------------------
switch_power_off:
		out	TCCR2, zero		; Disable PWM
		all_pFETs_off temp1
		all_nFETs_off temp1
		ret
;-----bko-----------------------------------------------------------------
; **** startup loop ****
init_startup:
		rcall	switch_power_off	; Disables PWM timer, turns off all FETs
		sei
		; RC pulse interrupt likely happens here
		cbr	flags1, (1<<EVAL_RC)	; Ignore any broken pulse from when interrupts were off
.if MOTOR_BRAKE
		nFET_brake temp1
.endif
wait_for_power_on:
		sbrs	flags1, EVAL_RC
		rjmp	wait_for_power_on
		rcall	evaluate_rc
		lds	YL, rc_duty_l
		lds	YH, rc_duty_h
		adiw	YL, 0			; Test for zero
		breq	wait_for_power_on
		ldi	temp1, RCP_TOT - 1	; allow some racing with t1ovfl_int
		cp	rc_timeout, temp1
		brcs	wait_for_power_on

start_from_running:
		rcall	switch_power_off
		comp_init temp1			; init comparator
		RED_off

		ldi	temp1, 27		; wait about 5mikosec
FETs_off_wt:	dec	temp1
		brne	FETs_off_wt

		ldi	YL, low(PWR_MIN_START)	; Start with limited power to
		ldi	YH, high(PWR_MIN_START) ; reduce the chance that we
		movw	sys_control_l, YL	; align to a timing harmonic

		sts	goodies, zero
		sbr	flags1, (1<<STARTUP)
		ldi	temp1, 4
		sts	timing_count, temp1
		rcall	start_timeout_start
		rcall	update_timing		; Clears POWER_OFF, sets duty, sets last_tcnt1
		rcall	start_timeout_start
		sts	timing_l, YL		; Reset timing to timeoutSTART
		sts	timing_h, YH
		sts	timing_x, temp1
		sts	timing_count, zero

		rcall	com5com6		; Enable pFET if not POWER_OFF
		rcall	com6com1		; Set comparator phase and nFET vector

		ldi	temp1, T2CLK
		out	TCCR2, temp1		; Enable PWM (ZL and XL have been set)

;-----bko-----------------------------------------------------------------
; **** running control loop ****

run1:		sbrc	flags1, REVERSE
		rjmp	run_reverse

run_forward:	rcall	wait_for_high
		rcall	com1com2
		rcall	wait_for_low
		rcall	com2com3
		rcall	wait_for_high
		rcall	com3com4
		rcall	wait_for_low
		rcall	com4com5
		rcall	wait_for_high
		rcall	com5com6
		rcall	wait_for_low
		rcall	com6com1
		rjmp	run6

run_to_start:	rcall	switch_power_off
		rjmp	start_from_running

run_reverse:	rcall	wait_for_low
		rcall	com1com6
		rcall	wait_for_high
		rcall	com6com5
		rcall	wait_for_low
		rcall	com5com4
		rcall	wait_for_high
		rcall	com4com3
		rcall	wait_for_low
		rcall	com3com2
		rcall	wait_for_high
		rcall	com2com1
run6:
		tst	rc_timeout
		breq	restart_control

.if MOTOR_BRAKE
		; Brake immediately whenever power is off
		sbrc	flags1, POWER_OFF
		rjmp	run_to_brake
.else
		; If timing is too slow and power is off, return to init_startup
		lds	temp1, timing_x
		cpi	temp1, byte3(TIMING_MIN*CPU_MHZ)
		sbrc	flags1, POWER_OFF
		brsh	run_to_brake
.endif
		cp	sys_control_l, zero
		cpc	sys_control_h, zero
		breq	run_to_start

		movw	YL, sys_control_l
		lds	temp1, goodies
		cpi	temp1, ENOUGH_GOODIES
		brcc	run6_2
		inc	temp1
		sts	goodies, temp1
		; Build up sys_control to PWR_MAX_START in steps.
		adiw	YL, ((PWR_MAX_START - PWR_MIN_START) + 15) / 16
		ldi	temp1, low (PWR_MAX_START)
		ldi	temp2, high(PWR_MAX_START)
		rjmp	run6_3

run6_2:		cbr	flags1, (1<<STARTUP)
		; Build up sys_control to MAX_POWER in steps.
		; If SLOW_THROTTLE is disabled, this only limits
		; initial start ramp-up; once running, sys_control
		; will stay at MAX_POWER unless timing is lost.
		adiw	YL, (POWER_RANGE + 31) / 32
		ldi	temp1, low (MAX_POWER)
		ldi	temp2, high(MAX_POWER)
run6_3:		cp	YL, temp1
		cpc	YH, temp2
		brcs	run6_4
		movw	sys_control_l, temp1
		rjmp	run1
run6_4:		movw	sys_control_l, YL
		rjmp	run1

restart_control:
		cli				; disable all interrupts
		rcall	switch_power_off
		rcall	wait30ms
		rcall	beep_f3
		rcall	beep_f2
		rcall	wait30ms
run_to_brake:	rjmp	init_startup

;-----bko-----------------------------------------------------------------
wait_timeout:	sts	goodies, zero
		sbr	flags1, (1<<STARTUP)
		ret
;-----bko-----------------------------------------------------------------
wait_for_low:	rcall	calc_next_timing_and_wait
		in	temp4, ACSR
		cbr	temp4, (1<<ACO)
		rjmp	wait_for_edge
;-----bko-----------------------------------------------------------------
wait_for_high:	rcall	calc_next_timing_and_wait
		in	temp4, ACSR
		sbr	temp4, (1<<ACO)
;-----bko-----------------------------------------------------------------
wait_for_edge:	cbr	flags0, (1<<OCT1B_PENDING)
wait_for_edge1:	lds	temp1, zc_filter_time
wait_for_edge2:	lds	temp2, zc_filter_time
wait_for_edge3:	sbrs	flags0, OCT1_PENDING
		rjmp	wait_timeout
		in	temp3, ACSR
		cp	temp3, temp4
		brne	wait_for_edge4
		cp	temp1, temp2		; Not yet crossed
		adc	temp1, zero		; Increment temp1 if < temp2
		sbrs	flags1, EVAL_RC
		rjmp	wait_for_edge3
		push	temp1
		rcall	evaluate_rc		; Does not clobber temp4
		pop	temp1
		rjmp	wait_for_edge2		; Restore temp2 and loop
wait_for_edge4:	dec	temp1			; Zero-cross has happened
		brne	wait_for_edge3		; Check again unless temp1 is zero
		sbrs	flags1, STARTUP
		rcall	set_com_timing
wait_for_edge5:	sbrc	flags1, EVAL_RC
		rcall	evaluate_rc
		sbrc	flags0, OCT1B_PENDING
		rjmp	wait_for_edge5		; Wait for commutation time
		ret
;-----bko-----------------------------------------------------------------
; *** commutation utilities ***
com1com2:	; Bp off, Ap on
		set_comp_phase_b temp1
		.if CnFET_port == BpFET_port
		BpFET_off_reg nfet_on
		BpFET_off_reg nfet_off
		.endif
		BpFET_off
		sbrc	flags1, POWER_OFF
		ret
		.if CnFET_port == ApFET_port
		ApFET_on_reg nfet_on
		ApFET_on_reg nfet_off
		.endif
		ApFET_on
		ret

com2com1:	; Bp on, Ap off
		set_comp_phase_a temp1
		.if CnFET_port == ApFET_port
		ApFET_off_reg nfet_on
		ApFET_off_reg nfet_off
		.endif
		ApFET_off
		sbrc	flags1, POWER_OFF
		ret
		.if CnFET_port == BpFET_port
		BpFET_on_reg nfet_on
		BpFET_on_reg nfet_off
		.endif
		BpFET_on
		ret

com2com3:	; Cn off, Bn on
		set_comp_phase_c temp1
		cli
		in	temp1, CnFET_port
		CnFET_off
		in	temp2, CnFET_port
		in	nfet_on, BnFET_port
		cpse	temp1, temp2
		BnFET_on
		sbrs	flags1, POWER_OFF
		BnFET_on_reg nfet_on
		mov	nfet_off, nfet_on
		sbrs	flags1, FULL_POWER
		BnFET_off_reg nfet_off
		ldi	XL, BnFET_port+0x20
		sei
		ret

com3com2:	; Cn on, Bn off
		set_comp_phase_b temp1
		cli
		in	temp1, BnFET_port
		BnFET_off
		in	temp2, BnFET_port
		in	nfet_on, CnFET_port
		cpse	temp1, temp2
		CnFET_on
		sbrs	flags1, POWER_OFF
		CnFET_on_reg nfet_on
		mov	nfet_off, nfet_on
		sbrs	flags1, FULL_POWER
		CnFET_off_reg nfet_off
		ldi	XL, CnFET_port+0x20
		sei
		ret

com3com4:	; Ap off, Cp on
		set_comp_phase_a temp1
		.if BnFET_port == ApFET_port
		ApFET_off_reg nfet_on
		ApFET_off_reg nfet_off
		.endif
		ApFET_off
		sbrc	flags1, POWER_OFF
		ret
		.if BnFET_port == CpFET_port
		CpFET_on_reg nfet_on
		CpFET_on_reg nfet_off
		.endif
		CpFET_on
		ret

com4com3:	; Ap on, Cp off
		set_comp_phase_c temp1
		.if BnFET_port == CpFET_port
		CpFET_off_reg nfet_on
		CpFET_off_reg nfet_off
		.endif
		CpFET_off
		sbrc	flags1, POWER_OFF
		ret
		.if BnFET_port == ApFET_port
		ApFET_on_reg nfet_on
		ApFET_on_reg nfet_off
		.endif
		ApFET_on
		ret

com4com5:	; Bn off, An on
		set_comp_phase_b temp1
		cli
		in	temp1, BnFET_port
		BnFET_off
		in	temp2, BnFET_port
		in	nfet_on, AnFET_port
		cpse	temp1, temp2
		AnFET_on
		sbrs	flags1, POWER_OFF
		AnFET_on_reg nfet_on
		mov	nfet_off, nfet_on
		sbrs	flags1, FULL_POWER
		AnFET_off_reg nfet_off
		ldi	XL, AnFET_port+0x20
		sei
		ret

com5com4:	; Bn on, An off
		set_comp_phase_a temp1
		cli
		in	temp1, AnFET_port
		AnFET_off
		in	temp2, AnFET_port
		in	nfet_on, BnFET_port
		cpse	temp1, temp2
		BnFET_on
		sbrs	flags1, POWER_OFF
		BnFET_on_reg nfet_on
		mov	nfet_off, nfet_on
		sbrs	flags1, FULL_POWER
		BnFET_off_reg nfet_off
		ldi	XL, BnFET_port+0x20
		sei
		ret

com5com6:	; Cp off, Bp on
		set_comp_phase_c temp1
		.if AnFET_port == CpFET_port
		CpFET_off_reg nfet_on
		CpFET_off_reg nfet_off
		.endif
		CpFET_off
		sbrc	flags1, POWER_OFF
		ret
		.if AnFET_port == BpFET_port
		BpFET_on_reg nfet_on
		BpFET_on_reg nfet_off
		.endif
		BpFET_on
		ret

com6com5:	; Cp on, Bp off
		set_comp_phase_b temp1
		.if AnFET_port == BpFET_port
		BpFET_off_reg nfet_on
		BpFET_off_reg nfet_off
		.endif
		BpFET_off
		sbrc	flags1, POWER_OFF
		ret
		.if AnFET_port == CpFET_port
		CpFET_on_reg nfet_on
		CpFET_on_reg nfet_off
		.endif
		CpFET_on
		ret

com6com1:	; An off, Cn on
		set_comp_phase_a temp1
		cli
		in	temp1, AnFET_port
		AnFET_off
		in	temp2, AnFET_port
		in	nfet_on, CnFET_port
		cpse	temp1, temp2
		CnFET_on
		sbrs	flags1, POWER_OFF
		CnFET_on_reg nfet_on
		mov	nfet_off, nfet_on
		sbrs	flags1, FULL_POWER
		CnFET_off_reg nfet_off
		ldi	XL, CnFET_port+0x20
		sei
		ret

com1com6:	; An on, Cn off
		set_comp_phase_c temp1
		cli
		in	temp1, CnFET_port
		CnFET_off
		in	temp2, CnFET_port
		in	nfet_on, AnFET_port
		cpse	temp1, temp2
		AnFET_on
		sbrs	flags1, POWER_OFF
		AnFET_on_reg nfet_on
		mov	nfet_off, nfet_on
		sbrs	flags1, FULL_POWER
		AnFET_off_reg nfet_off
		ldi	XL, AnFET_port+0x20
		sei
		ret

.exit
